#include <TIPbusTransaction.h>

///////////////////////////////////////////////////////////////////////////////
/**constructor.
*/
TIPbusTransaction::TIPbusTransaction(int int_size)
{
maxPayloadLength=int_size;
WORD=new quint32[maxPayloadLength];
buf_ptr=new quint32*[maxPayloadLength/2];
}
///////////////////////////////////////////////////////////////////////////////
/**clean.
*/
void TIPbusTransaction::clean()
{
currentTxLength=0;
expectedRxLength=0;
transactionCount=0;
IDSet=false;
}
///////////////////////////////////////////////////////////////////////////////
/**destructor.
*/
TIPbusTransaction::~TIPbusTransaction()
{
delete [] WORD;
delete [] buf_ptr;
}
//////////////////////////////////////////////////////////////////////////////
/**get expected return frame size.
*/
uint TIPbusTransaction::getExpectedRxLength()
{
return sizeof(quint32)*expectedRxLength;
}
///////////////////////////////////////////////////////////////////////////////
/**get avaliable RX frame size.
When creating request, it must be checked that the expected answer still fits in the 
returning payload.
*/
int TIPbusTransaction::getAvailableRxLength()
{
return maxPayloadLength-expectedRxLength;
}
///////////////////////////////////////////////////////////////////////////////
/**.
When creating request, it must be checked that the additional transaction will fit in the
payload.
*/
int TIPbusTransaction::getAvailableTxLength()
{
return maxPayloadLength-currentTxLength;
}
///////////////////////////////////////////////////////////////////////////////
/**Get the current data frame size in byte (4 bytes).
*/
uint TIPbusTransaction::getFrameSize()
{
return currentTxLength*sizeof(quint32);
}
///////////////////////////////////////////////////////////////////////////////
quint32 TIPbusTransaction::expectedPacketID()
{
return packetID;
}
///////////////////////////////////////////////////////////////////////////////
uint TIPbusTransaction::getTransactionCount()
{
return transactionCount;
}
///////////////////////////////////////////////////////////////////////////////
/**Set next packet ID.
 And initialize frame
*/
bool TIPbusTransaction::setPacketID(quint32 id)
{
if (!IDSet)
	{
	packetID=id;
	IDSet=true;
	currentTxLength+=1;
	expectedRxLength+=1;
	WORD[0]=(0x20<<24) | (packetID<<8) | 0xF0; // Ipbus packet header
	}

return true;
}
///////////////////////////////////////////////////////////////////////////////
/**pointer to data buffer.
*/
quint32 *TIPbusTransaction::dataFrame_ptr()
{
return WORD;
}
///////////////////////////////////////////////////////////////////////////////
/**pointer to transaction data buffer.
*/
quint32 *TIPbusTransaction::transactionPtr_ptr(int trans)
{
return buf_ptr[trans];
}
///////////////////////////////////////////////////////////////////////////////
bool TIPbusTransaction::appendToTransaction(int packID,TreqParam &param)
{
switch(param.type)
	{
	case _read_std:
	case _read_ni:
		if (!setPacketID(packID)) return false;

		if (getAvailableTxLength()<2) return false;//min is hdr+ baseadd
		if (getAvailableRxLength()<2) return false; //min is hdr + one data

		//cout << dec << "getAvailableRxLength()=" << getAvailableRxLength() << endl;
		if (getAvailableRxLength()>param.param1 || getAvailableRxLength()>255)
			{
			if (param.param1<=255)
				{
				readTransaction(/*type*/param.type,/*add*/param.baseadd,/*buff*/param.ptr,/*size*/param.param1);
				param.type=_done;
				return true;
				}
			else
				{
				int usedSize=255;
				readTransaction(/*type*/param.type,/*add*/param.baseadd,/*buff*/param.ptr,/*size*/usedSize);
				if (param.type==_read_std)
					param.baseadd+=usedSize;
				param.ptr+=usedSize;
				param.param1-=usedSize;
				return true;
				}
			}
		else
			{
			int usedSize=getAvailableRxLength()-1;
			readTransaction(/*type*/param.type,/*add*/param.baseadd,/*buff*/param.ptr,/*size*/usedSize);
			if (param.type==_read_std)
				param.baseadd+=usedSize;
			param.ptr+=usedSize;
			param.param1-=usedSize;
			return false;
			}

	case _rmw_bit:
		if (!setPacketID(packID)) return false;

		if (getAvailableTxLength()<4) return false;//min is hdr+ baseadd+AND term + OR term
		if (getAvailableRxLength()<2) return false; //min is hdr + one data
		
		expectedRxLength+=2;

		//save pointer
		buf_ptr[transactionCount]=param.ptr;

		//build frame
		WORD[currentTxLength]=(0x2<<28) | (transactionCount<<16) | (1<<8) | (_rmw_bit<<4) | 0xF; //IPBUS transaction header
		currentTxLength++;

		WORD[currentTxLength]=param.baseadd;//base address
		currentTxLength++;

		WORD[currentTxLength]=param.param1;//ANDterm
		currentTxLength++;

		WORD[currentTxLength]=param.param2;//ORterm
		currentTxLength++;

		transactionCount++;
		param.type=_done;
		return true;

	case _rmw_sum:
		if (!setPacketID(packID)) return false;

		if (getAvailableTxLength()<3) return false;//min is hdr+ baseadd+AND term + OR term
		if (getAvailableRxLength()<2) return false; //min is hdr + one data
		
		expectedRxLength+=2;

		//save pointer
		buf_ptr[transactionCount]=param.ptr;

		//build frame
		WORD[currentTxLength]=(0x2<<28) | (transactionCount<<16) | (1<<8) | (_rmw_sum<<4) | 0xF; //IPBUS transaction header
		currentTxLength++;

		WORD[currentTxLength]=param.baseadd;//base address
		currentTxLength++;

		WORD[currentTxLength]=param.param1;//ADDEND
		currentTxLength++;

		transactionCount++;
		param.type=_done;
		return true;


	case _write_std:
	case _write_ni:
		if (!setPacketID(packID)) return false;

		if (getAvailableTxLength()<3) return false; //min is hdr+ baseadd+ one data
		if (getAvailableRxLength()<1) return false; //acknowledge 1 hdr only

		//cout << dec << "getAvailableRxLength()=" << getAvailableRxLength() << endl;
		if (getAvailableTxLength()>(param.param1+2) || getAvailableTxLength()>(255+2)) //max trans size +baseadd + hdr
			{
			if (param.param1<=255)
				{
				writeTransaction(/*type*/param.type,/*add*/param.baseadd,/*buff*/param.ptr,/*size*/param.param1);
				param.type=_done;
				return true;
				}
			else
				{
				int usedSize=255;
				writeTransaction(/*type*/param.type,/*add*/param.baseadd,/*buff*/param.ptr,/*size*/usedSize);
				if (param.type==_write_std)
					param.baseadd+=usedSize;
				param.ptr+=usedSize;
				param.param1-=usedSize;
				return true;
				}
			}
		else
			{
			int usedSize=getAvailableTxLength()-2;
			writeTransaction(/*type*/param.type,/*add*/param.baseadd,/*buff*/param.ptr,/*size*/usedSize);
			if (param.type==_write_std)
				param.baseadd+=usedSize;
			param.ptr+=usedSize;
			param.param1-=usedSize;
			return false;
			}

	case _dispatch:
		param.type=_done;
		return false;

	default :
		return false;
	}
}
///////////////////////////////////////////////////////////////////////////////
/**Read transaction implementation.
*/
void TIPbusTransaction::readTransaction(int type,int add, quint32 *buff,int size)
{
//cout << "readtransaction append beg=" << dec << currentTxLength<< endl;
//cout << "readtransaction append beg=" <<  dec << expectedRxLength<< endl;
expectedRxLength+=(size+1);

//save pointer
buf_ptr[transactionCount]=buff;

//build frame
WORD[currentTxLength]=(0x2<<28) | (transactionCount<<16) | (size<<8) | (type<<4) | 0xF; //IPBUS transaction header
currentTxLength++;

WORD[currentTxLength]=add;//base address
currentTxLength++;

transactionCount++;
//cout << "readtransaction append end=" << currentTxLength<< endl;
//cout << "readtransaction append end=" <<  dec << expectedRxLength<< endl;
}
///////////////////////////////////////////////////////////////////////////////
/**Write transaction implementation.
*/
void TIPbusTransaction::writeTransaction(int type,int add, quint32 *buff,int size)
{
//cout << "readtransaction append beg=" << dec << currentTxLength<< endl;
//cout << "readtransaction append beg=" <<  dec << expectedRxLength<< endl;
expectedRxLength+=1;

//build frame
WORD[currentTxLength]=(0x2<<28) | (transactionCount<<16) | (size<<8) | (type<<4) | 0xF; //IPBUS transaction header
currentTxLength++;

WORD[currentTxLength]=add;//base address
currentTxLength++;

for (int i=0;i<size;i++)
	{
	WORD[currentTxLength]=buff[i];
	currentTxLength++;
	}
transactionCount++;
//cout << "readtransaction append end=" << currentTxLength<< endl;
//cout << "readtransaction append end=" <<  dec << expectedRxLength<< endl;
}
