-- ntp add_sub
--
--Pipelined  Adder/subtractorfor 64 bit operation
--
-- Olivier Bourrion, May 2015

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity ntp_addsub is 
generic(
	is_adder : boolean :=true;
	size : natural :=64 -- must be even number here
	);
port(
	clk : in std_logic;

	op_1 : in std_logic_vector(size-1 downto 0);
	op_2 : in std_logic_vector(size-1 downto 0);

	result : out std_logic_vector(size-1 downto 0)
	);
end entity ntp_addsub;

architecture RTL of ntp_addsub is
signal op_1_LSB : unsigned(size/2 downto 0);
signal op_1_MSB : unsigned(size/2 downto 0);

signal op_2_LSB : unsigned(size/2 downto 0);
signal op_2_MSB : unsigned(size/2 downto 0);

signal LSB_result : unsigned(size/2 downto 0);
signal LSB_result_1ck : unsigned(size/2 downto 0);
signal MSB_result : unsigned(size/2 downto 0);
signal MSB_result_1ck : unsigned(size/2 downto 0);

begin
op_1_LSB<='0' & unsigned(op_1(size/2-1 downto 0));
op_1_MSB<='0' & unsigned(op_1(size-1 downto size/2));

op_2_LSB<='0' & unsigned(op_2(size/2-1 downto 0));
op_2_MSB<='0' & unsigned(op_2(size-1 downto size/2));

process(clk)
begin
	if rising_edge(clk) then
		-- first stage
		if is_adder then
			LSB_result<=op_1_LSB+op_2_LSB;
			MSB_result<=op_1_MSB+op_2_MSB;
		else
			LSB_result<=op_1_LSB-op_2_LSB;
			MSB_result<=op_1_MSB-op_2_MSB;
		end if;
		
		-- second stage
		LSB_result_1ck<=LSB_result;
		if LSB_result(size/2)='1' then -- carry propagation
			if is_adder then
				MSB_result_1ck<=MSB_result+1;
			else
				MSB_result_1ck<=MSB_result-1;
			end if;
		else
			MSB_result_1ck<=MSB_result;
		end if;

	end if;
end process;

result(size/2-1 downto 0)   <=std_logic_vector(LSB_result_1ck(size/2-1 downto 0));
result(size-1 downto size/2)<=std_logic_vector(MSB_result_1ck(size/2-1 downto 0));
end RTL;
