-- ntp dds
--
--
-- DDS to build sub second NTP clock from a random reference clock frequency
-- natural fluctuation/jitter of sub_sec_clock is expected
--
-- output clock un fout=refclk*P/Q
--
-- sample configurations:
-- refclk=10MHz, log_P=20 -> P= 2^20,Q=10M, fout=1,048,576Hz
--
-- No need to try to be better than 1 us, seems to be the best time resolution of NTP
-- 
-- Olivier Bourrion, May 2015

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity ntp_dds is 
generic(
	log_P: natural := 20; -- P is computed as 2**log_P
	Q: natural :=10E6
	);
port(
	refclk: in std_logic; -- Stable reference clock
	rst: in std_logic; -- rst

	upd_dds : in std_logic;
	freq_adj : in std_logic_vector(log_P-1 downto 0);
    
	sub_sec_top: out std_logic
	);
end ntp_dds;

architecture RTL of ntp_dds is
constant P_init : natural :=2**log_P;
signal dds_cnt : natural range 0 to Q;
signal P : natural range 0 to Q;
signal margin : natural range 0 to Q;
signal delta_f : integer range -P_init/2 to P_init/2-1;

begin
delta_f<=conv_integer(signed(freq_adj));
-------------------------------------------------------------------------------
-- frequency generator
-------------------------------------------------------------------------------
process(refclk)
begin
	if rising_edge(refclk) then
		if (rst='1') then
			sub_sec_top<='0';
			margin<=Q;
			P<=P_init;
			dds_cnt<=0;
		else
			if (margin>P) then
				dds_cnt<=dds_cnt+P;
				margin<=margin-P;
				sub_sec_top<='0';
			else
				sub_sec_top<='1';
				dds_cnt<=P-margin;
				margin<=Q-(P-margin);
			end if;

			if upd_dds='1' then
				P<=P+delta_f;
			end if;
		end if;
	end if;
end process;
end RTL;
