library IEEE;
use IEEE.Std_Logic_1164.all;
use ieee.numeric_std.all;
 
entity tb_serial_unsigned_division is
generic(
	dividend_size : natural :=7;
	divisor_size : natural :=4
	);
end entity tb_serial_unsigned_division;


architecture simu of tb_serial_unsigned_division is
component serial_unsigned_division is
generic(
	dividend_size : natural :=32;
	divisor_size : natural :=4
	);
port(
	clk : in  std_logic;
	reset : in  std_logic;
	load  : in  std_logic;
	dividend : in  unsigned(dividend_size-1 downto 0);
	divisor : in  unsigned(divisor_size-1 downto 0);

	ready : out std_logic;
	quotient : out unsigned(dividend_size-1 downto 0);
	remain : out unsigned(dividend_size-1 downto 0)
	);
end component serial_unsigned_division;
 
signal clk :  std_logic;
signal reset :  std_logic;
signal load  :  std_logic;
signal dividend :  unsigned(dividend_size-1 downto 0);
signal divisor :  unsigned(divisor_size-1 downto 0);

signal cnt : natural;
signal idx : natural;

begin
-------------------------------------------------------------------------------
--
-------------------------------------------------------------------------------
process
begin
	clk<='0';
	wait for 5 ns;

	clk<='1';
	wait for 5 ns;
end process;

process(reset,clk)
begin
	if reset='1' then
		load<='0';
		dividend<=to_unsigned(0,dividend_size);
		divisor<=to_unsigned(0,divisor_size);
		cnt<=0;
		idx<=0;
	elsif rising_edge(clk) then
		if cnt=63 then
			load<='1';
			cnt<=0;
			idx<=idx+1;
			case idx is
				when 0 =>
					dividend<=to_unsigned(53,dividend_size);
					divisor<=to_unsigned(5,divisor_size);
				when 1 =>
					dividend<=to_unsigned(6,dividend_size);
					divisor<=to_unsigned(15,divisor_size);
				when 2 =>
					dividend<=to_unsigned(53,dividend_size);
					divisor<=to_unsigned(1,divisor_size);
				when 3 =>
					dividend<=to_unsigned(70,dividend_size);
					divisor<=to_unsigned(8,divisor_size);
				when 4 =>
					dividend<=to_unsigned(55,dividend_size);
					divisor<=to_unsigned(55,divisor_size);
				when 5 =>
					dividend<=to_unsigned(71,dividend_size);
					divisor<=to_unsigned(2,divisor_size);
				when others =>
					dividend<=to_unsigned(53,dividend_size);
					divisor<=to_unsigned(2,divisor_size);
			end case;
		else
			load<='0';
			cnt<=cnt+1;
		end if;
	end if;
end process;
-------------------------------------------------------------------------------
-- DUT
-------------------------------------------------------------------------------
DUT :  serial_unsigned_division generic map(
	dividend_size=>dividend_size,
	divisor_size=>divisor_size
	)
port map(
	clk=>clk,
	reset=>reset,
	load=>load,
	dividend=>dividend,
	divisor=>divisor,

	ready=>open,
	quotient=>open,
	remain=>open
	);
-------------------------------------------------------------------------------
-- main
-------------------------------------------------------------------------------
process
begin
	reset<='1';
	wait for 50 ns;
	
	reset<='0';
	wait for 5 us;

	report "fin de simulation" severity error;
end process;
end simu;
