library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_arith.ALL;
use IEEE.STD_LOGIC_unsigned.ALL;
use work.pack_axi.all;
use work.pack_ipv4_types.all;
use work.pack_arp_types.all;

entity UDP_Complete is
generic (
	USE_RX_FIFO     : boolean :=true; --only if gmii_rx_clk /= clk125
	IN_SIMULATION     : boolean :=false;
	SIMUL_DST_MAC_NUMBER  : std_logic_vector(47 downto 0):=x"AAAA_BBBB_CCCC";
	CLOCK_FREQ			: integer := 125000000;							-- freq of data_in_clk -- needed to timout cntr
	ARP_TIMEOUT			: integer := 60;									-- ARP response timeout (s)
	ARP_MAX_PKT_TMO	: integer := 5;									-- # wrong nwk pkts received before set error
	MAX_ARP_ENTRIES 	: integer := 255									-- max entries in the ARP store
	);
Port (
	-- UDP TX signals
	udp_tx_start			: in std_logic;							-- indicates req to tx UDP
	udp_txi					: in udp_tx_type;							-- UDP tx cxns
	udp_tx_result			: out std_logic_vector (1 downto 0);-- tx status (changes during transmission)
	udp_tx_data_out_ready: out std_logic;							-- indicates udp_tx is ready to take data

	-- UDP RX signals
	udp_rx_start			: out std_logic;							-- indicates receipt of udp header
	udp_rxo					: out udp_rx_type;

	-- IP RX signals
	ip_rx_hdr				: out ipv4_rx_header_type;

	-- system signals
	clk125					: in  STD_LOGIC;
	reset 					: in  STD_LOGIC;
	our_ip_address 		: in STD_LOGIC_VECTOR (31 downto 0);
	our_mac_address 		: in std_logic_vector (47 downto 0);
	control					: in udp_control_type;

	-- status signals
	arp_pkt_count			: out STD_LOGIC_VECTOR(7 downto 0);			-- count of arp pkts received
	ip_pkt_count			: out STD_LOGIC_VECTOR(7 downto 0);			-- number of IP pkts received for us
	
	-- gmii TX interface
	-- 	gmii_gtx_clk : out STD_LOGIC; -- must be generated with appropriate system
	gmii_tx_en : out STD_LOGIC;
	gmii_tx_er : out STD_LOGIC;
	gmii_txd : out STD_LOGIC_VECTOR(7 downto 0);

	-- gmii RX interface
	gmii_rx_clk: in STD_LOGIC;
   	gmii_rx_dv: in STD_LOGIC;
	gmii_rx_er: in STD_LOGIC;
	gmii_rxd : in STD_LOGIC_VECTOR(7 downto 0)
	);
end UDP_Complete;

architecture RTL of UDP_Complete is
component UDP_Complete_nomac
	 generic (
			IN_SIMULATION     : boolean :=false;
			SIMUL_DST_MAC_NUMBER  : std_logic_vector(47 downto 0):=x"AAAA_BBBB_CCCC";
			CLOCK_FREQ			: integer := 125000000;							-- freq of data_in_clk -- needed to timout cntr
			ARP_TIMEOUT			: integer := 60;									-- ARP response timeout (s)
			ARP_MAX_PKT_TMO	: integer := 5;									-- # wrong nwk pkts received before set error
			MAX_ARP_ENTRIES 	: integer := 255									-- max entries in the ARP store
			);
    Port (
			-- UDP TX signals
			udp_tx_start			: in std_logic;							-- indicates req to tx UDP
			udp_txi					: in udp_tx_type;							-- UDP tx cxns
			udp_tx_result			: out std_logic_vector (1 downto 0);-- tx status (changes during transmission)
			udp_tx_data_out_ready: out std_logic;							-- indicates udp_tx is ready to take data
			-- UDP RX signals
			udp_rx_start			: out std_logic;							-- indicates receipt of udp header
			udp_rxo					: out udp_rx_type;
			-- IP RX signals
			ip_rx_hdr				: out ipv4_rx_header_type;
			-- system signals
			rx_clk					: in  STD_LOGIC;
			tx_clk					: in  STD_LOGIC;
			reset 					: in  STD_LOGIC;
			our_ip_address 		: in STD_LOGIC_VECTOR (31 downto 0);
			our_mac_address 		: in std_logic_vector (47 downto 0);
			control					: in udp_control_type;
			-- status signals
			arp_pkt_count			: out STD_LOGIC_VECTOR(7 downto 0);			-- count of arp pkts received
			ip_pkt_count			: out STD_LOGIC_VECTOR(7 downto 0);			-- number of IP pkts received for us
			-- MAC Transmitter
			mac_tx_tdata         : out  std_logic_vector(7 downto 0);	-- data byte to tx
			mac_tx_tvalid        : out  std_logic;							-- tdata is valid
			mac_tx_tready        : in std_logic;							-- mac is ready to accept data
			mac_tx_terror        : out  std_logic;							-- tells mac to produce an error
			mac_tx_tfirst        : out  std_logic;							-- indicates first byte of frame
			mac_tx_tlast         : out  std_logic;							-- indicates last byte of frame
			-- MAC Receiver
			mac_rx_tdata         : in std_logic_vector(7 downto 0);	-- data byte received
			mac_rx_tvalid        : in std_logic;							-- indicates tdata is valid
			mac_rx_tready        : out  std_logic;							-- tells mac that we are ready to take data
			mac_rx_terror        : in  std_logic;							-- tells mac produced an error
			mac_rx_tlast         : in std_logic								-- indicates last byte of the trame
			);
end component;
component tri_mode_eth_mac
port (
	glbl_rstn : in std_logic;
	rx_axi_rstn : in std_logic;
	tx_axi_rstn : in std_logic;
	rx_axi_clk : in std_logic;
	rx_reset_out : out std_logic;
	rx_axis_mac_tdata : out std_logic_vector(7 downto 0);
	rx_axis_mac_tvalid : out std_logic;
	rx_axis_mac_tlast : out std_logic;
	rx_axis_mac_tuser : out std_logic;
	rx_statistics_vector : out std_logic_vector(27 downto 0);
	rx_statistics_valid : out std_logic;
	tx_axi_clk : in std_logic;
	tx_reset_out : out std_logic;
	tx_axis_mac_tdata : in std_logic_vector(7 downto 0);
	tx_axis_mac_tvalid : in std_logic;
	tx_axis_mac_tlast : in std_logic;
	tx_axis_mac_tuser : in std_logic;
	tx_axis_mac_tready : out std_logic;
	tx_ifg_delay : in std_logic_vector(7 downto 0);--! Intrame gap delay extension-> unused
	tx_statistics_vector : out std_logic_vector(31 downto 0);
	tx_statistics_valid : out std_logic;
	pause_req : in std_logic;
	pause_val : in std_logic_vector(15 downto 0);
	speed_is_100 : out std_logic;
	speed_is_10_100 : out std_logic;
	gmii_txd : out std_logic_vector(7 downto 0);
	gmii_tx_en : out std_logic;
	gmii_tx_er : out std_logic;
	gmii_rxd : in std_logic_vector(7 downto 0);
	gmii_rx_dv : in std_logic;
	gmii_rx_er : in std_logic;
	rx_mac_config_vector : in std_logic_vector(79 downto 0);
	tx_mac_config_vector : in std_logic_vector(79 downto 0)
	);
end component;
-- component mac_fifo_axi4
component mac_fifo_axi4_lpsc
  port (
	 m_aclk : in std_logic;
	 s_aclk : in std_logic;
	 s_aresetn : in std_logic;
	 s_axis_tvalid : in std_logic;
	 s_axis_tready : out std_logic;
	 s_axis_tdata : in std_logic_vector(7 downto 0);
	 s_axis_tlast : in std_logic;
	 s_axis_tuser : in std_logic_vector(0 downto 0);
	 m_axis_tvalid : out std_logic;
	 m_axis_tready : in std_logic;
	 m_axis_tdata : out std_logic_vector(7 downto 0);
	 m_axis_tlast : out std_logic;
	 m_axis_tuser : out std_logic_vector(0 downto 0)
  );
end component;

signal rstn : std_logic;

signal rx_mac_config_vector : std_logic_vector(79 downto 0);
signal tx_mac_config_vector : std_logic_vector(79 downto 0);

-- MAC Transmitter
signal mac_tx_tdata         :  std_logic_vector(7 downto 0);	-- data byte to tx
signal mac_tx_tvalid        :  std_logic;							-- tdata is valid
signal mac_tx_tready        : std_logic;							-- mac is ready to accept data
signal mac_tx_terror        : std_logic;
signal mac_tx_tlast         :  std_logic;							--dicates last byte of frame
-- MAC Receiver
signal mac_rx_tdata         : std_logic_vector(7 downto 0);	-- data byte received
signal mac_rx_tvalid        : std_logic;							--dicates tdata is valid
signal mac_rx_tready        : std_logic;					
signal mac_rx_tlast         : std_logic;
signal mac_rx_terror        : std_logic_vector(0 downto 0);

signal rx_data_e: std_logic_vector(7 downto 0);
signal rx_valid_e, rx_last_e,rx_rst : std_logic;
signal rx_user_ef: std_logic_vector(0 downto 0);
	
	
begin

lbl_UDP : UDP_Complete_nomac generic map(
			IN_SIMULATION=>IN_SIMULATION,
			SIMUL_DST_MAC_NUMBER=>SIMUL_DST_MAC_NUMBER,

			CLOCK_FREQ		=> CLOCK_FREQ,
			ARP_TIMEOUT		=> ARP_TIMEOUT,
			ARP_MAX_PKT_TMO	=> ARP_MAX_PKT_TMO,
			MAX_ARP_ENTRIES =>MAX_ARP_ENTRIES 
			)
    port map(
			-- UDP TX signals
			udp_tx_start=>udp_tx_start,
			udp_txi=>udp_txi,
			udp_tx_result=>udp_tx_result,
			udp_tx_data_out_ready=>udp_tx_data_out_ready,
			
			-- UDP RX signals
			udp_rx_start=>udp_rx_start,
			udp_rxo=>udp_rxo,
			
			-- IP RX signals
			ip_rx_hdr=>ip_rx_hdr,
			
			-- system signals
			rx_clk=>clk125,
			tx_clk=>clk125,
			reset=>reset,
			our_ip_address=>our_ip_address,
			our_mac_address=>our_mac_address,
			control=>control,

			-- status signals
			arp_pkt_count=>arp_pkt_count,
			ip_pkt_count=>ip_pkt_count,

			-- MAC Transmitter
			mac_tx_tdata=>mac_tx_tdata,
			mac_tx_tvalid=>mac_tx_tvalid,
			mac_tx_tready=>mac_tx_tready,
			mac_tx_terror=>mac_tx_terror,
			mac_tx_tfirst=>open,
			mac_tx_tlast=>mac_tx_tlast,
			-- MAC Receiver
			mac_rx_tdata=>mac_rx_tdata,
			mac_rx_tvalid=>mac_rx_tvalid,
			mac_rx_terror=>mac_rx_terror(0),
			mac_rx_tready=>mac_rx_tready,
			mac_rx_tlast=>mac_rx_tlast
			);

rstn <= not reset;

-- process(gmii_rx_clk)
-- begin
-- 	if rising_edge(gmii_rx_clk) then
		rx_rst <= not reset;
-- 	end if;
-- end process;

lbl_use_fifo : if USE_RX_FIFO generate
	-- in that case gmii_rx_clk /= clk125
	-- fifo: mac_fifo_axi4 port map(
	fifo: mac_fifo_axi4_lpsc port map(
		s_aclk => gmii_rx_clk,
		s_aresetn => rx_rst,
		s_axis_tvalid => rx_valid_e,
		s_axis_tready => open,
		s_axis_tdata => rx_data_e,
		s_axis_tlast => rx_last_e,
		s_axis_tuser => rx_user_ef,
		m_aclk => clk125,
		m_axis_tvalid => mac_rx_tvalid,
		m_axis_tready => '1',
		m_axis_tdata => mac_rx_tdata,
		m_axis_tlast => mac_rx_tlast,
		m_axis_tuser => mac_rx_terror
	); -- Clock domain crossing FIFO
end generate lbl_use_fifo;

lbl_bypass_fifo : if not(USE_RX_FIFO) generate
	-- in that case gmii_rx_clk == clk125
	mac_rx_tdata<=rx_data_e;
	mac_rx_tvalid<=rx_valid_e;
	mac_rx_terror<=rx_user_ef;
	mac_rx_tlast<=rx_last_e;
end generate lbl_bypass_fifo;


lbl_MAC:  tri_mode_eth_mac port map (
	glbl_rstn=>rstn,
	rx_axi_rstn=>rstn,
	tx_axi_rstn=>rstn,
	rx_axi_clk => gmii_rx_clk,
	rx_reset_out=>open,
	rx_axis_mac_tdata=>rx_data_e,
	rx_axis_mac_tvalid=>rx_valid_e,
	rx_axis_mac_tlast=>rx_last_e,
	rx_axis_mac_tuser=>rx_user_ef(0),
	rx_statistics_vector=>open,
	rx_statistics_valid=>open,
	tx_axi_clk => clk125,
	tx_reset_out=>open,
	tx_axis_mac_tdata=>	mac_tx_tdata,
	tx_axis_mac_tvalid=>mac_tx_tvalid,
	tx_axis_mac_tlast=>mac_tx_tlast,
	tx_axis_mac_tuser=>mac_tx_terror,
	tx_axis_mac_tready=>mac_tx_tready,
	tx_ifg_delay=>X"00",
	tx_statistics_vector=>open,
	tx_statistics_valid=>open,
	pause_req=>'0',
	pause_val=>X"0000",
	speed_is_100=>open,
	speed_is_10_100=>open,
	gmii_txd => gmii_txd ,
	gmii_tx_en => gmii_tx_en ,
	gmii_tx_er => gmii_tx_er ,

	gmii_rxd =>gmii_rxd ,
	gmii_rx_dv=>gmii_rx_dv,
	gmii_rx_er=>gmii_rx_er,

	rx_mac_config_vector=>rx_mac_config_vector,
	tx_mac_config_vector=>tx_mac_config_vector
	);
rx_mac_config_vector <= X"0000_0000_0000_0000_0812";
tx_mac_config_vector <= X"0000_0000_0000_0000_0012";

end RTL;
